@@ -9,10 +9,13 @@ import os.path
 CONFIGURATION_DIR = (os.environ.get('XDG_CONFIG_HOME') or
 	os.path.expanduser('~/.config'))
 MATHJAX_LOCAL_URL = 'file:///usr/share/javascript/mathjax/MathJax.js'
-MATHJAX_WEB_URL = 'http://cdn.mathjax.org/mathjax/latest/MathJax.js'
+MATHJAX_WEB_URL = 'https://cdn.mathjax.org/mathjax/latest/MathJax.js'
 
 PYGMENTS_STYLE = 'default'
 
+class MarkupsWarning(RuntimeWarning):
+	pass
+
 def get_pygments_stylesheet(selector, style=None):
 	if style is None:
 		style = PYGMENTS_STYLE
