@@ -13,7 +13,7 @@ class TextileMarkup(AbstractMarkup):
 	"""
 	name = 'Textile'
 	attributes = {
-		common.LANGUAGE_HOME_PAGE: 'http://textile.sitemonks.com/',
+		common.LANGUAGE_HOME_PAGE: 'http://en.wikipedia.org/wiki/Textile_(markup_language)',
 		common.MODULE_HOME_PAGE: 'https://github.com/sebix/python-textile',
 		common.SYNTAX_DOCUMENTATION: 'http://movabletype.org/documentation/author/textile-2-syntax.html'
 	}
