@@ -107,6 +107,27 @@ mathjax_multiline_output = \
 </p>
 '''
 
+mathjax_multilevel_source = \
+r'''
+\begin{equation*}
+  \begin{pmatrix}
+    1 & 0\\
+    0 & 1
+  \end{pmatrix}
+\end{equation*}
+'''
+
+mathjax_multilevel_output = \
+r'''<p>
+<script type="math/tex; mode=display">\begin{equation*}
+  \begin{pmatrix}
+    1 & 0\\
+    0 & 1
+  \end{pmatrix}
+\end{equation*}</script>
+</p>
+'''
+
 @unittest.skipUnless(MarkdownMarkup.available(), 'Markdown not available')
 class MarkdownTest(unittest.TestCase):
 	maxDiff = None
@@ -117,10 +138,25 @@ class MarkdownTest(unittest.TestCase):
 
 	def test_extensions_loading(self):
 		markup = MarkdownMarkup()
-		self.assertFalse(markup._check_extension_exists('nonexistent'))
-		self.assertFalse(markup._check_extension_exists('nonexistent(someoption)'))
-		self.assertTrue(markup._check_extension_exists('meta'))
-		self.assertTrue(markup._check_extension_exists('meta(someoption)'))
+		self.assertIsNone(markup._canonicalize_extension_name('nonexistent'))
+		self.assertIsNone(markup._canonicalize_extension_name('nonexistent(someoption)'))
+		self.assertIsNone(markup._canonicalize_extension_name('.foobar'))
+		self.assertEqual(markup._canonicalize_extension_name('meta'), 'markdown.extensions.meta')
+		self.assertEqual(markup._canonicalize_extension_name('meta(someoption)'),
+			'markdown.extensions.meta(someoption)')
+
+	def test_loading_extensions_by_module_name(self):
+		markup = MarkdownMarkup(extensions=['markdown.extensions.footnotes'])
+		source = ('Footnotes[^1] have a label and the content.\n\n'
+		          '[^1]: This is a footnote content.')
+		html = markup.get_document_body(source)
+		self.assertIn('<sup', html)
+		self.assertIn('footnote-backref', html)
+
+	def test_removing_duplicate_extensions(self):
+		markup = MarkdownMarkup(extensions=['remove_extra', 'toc', 'markdown.extensions.toc'])
+		self.assertEqual(len(markup.extensions), 1)
+		self.assertIn('markdown.extensions.toc', markup.extensions)
 
 	def test_extensions_parameters(self):
 		markup = MarkdownMarkup(extensions=['toc(anchorlink=1)'])
@@ -128,6 +164,13 @@ class MarkdownTest(unittest.TestCase):
 		self.assertEqual(html,
 			'<h2 id="header"><a class="toclink" href="#header">Header</a></h2>\n')
 
+	def test_document_extensions_parameters(self):
+		markup = MarkdownMarkup(extensions=[])
+		toc_header = '<!--- Required extensions: toc(anchorlink=1) --->\n\n'
+		html = markup.get_document_body(toc_header + '## Header')
+		self.assertEqual(html, toc_header +
+			'<h2 id="header"><a class="toclink" href="#header">Header</a></h2>\n')
+
 	def test_extra(self):
 		markup = MarkdownMarkup()
 		html = markup.get_document_body(tables_source)
@@ -187,5 +230,16 @@ class MarkdownTest(unittest.TestCase):
 		body = markup.get_document_body(mathjax_multiline_source)
 		self.assertEqual(mathjax_multiline_output, body)
 
+	def test_mathjax_multilevel(self):
+		markup = MarkdownMarkup()
+		body = markup.get_document_body(mathjax_multilevel_source)
+		self.assertEqual(mathjax_multilevel_output, body)
+
+	@unittest.skipUnless(hasattr(unittest.TestCase, 'assertWarnsRegex'), 'assertWarnsRegex is not supported')
+	def test_not_loading_sys(self):
+		with self.assertWarnsRegex(ImportWarning, 'Extension "sys" does not exist.'):
+			markup = MarkdownMarkup(extensions=['sys'])
+		self.assertNotIn('sys', markup.extensions)
+
 if __name__ == '__main__':
 	unittest.main()
